"""
SUBSTITUTION REPLACEMENT 'UV_STICKY_SELECT_MODE_REF':
(bpy.types.ToolSettings.uv_sticky_select_mode which may be passed in directly).

SUBSTITUTION REPLACEMENT 'UV_STICKY_SELECT_MODE_TYPE':
Literal['SHARED_LOCATION', 'DISABLED', 'SHARED_VERTEX']

SUBSTITUTION REPLACEMENT 'UV_SELECT_FLUSH_MODE_NEEDED':
This function is selection-mode independent, typically bmesh.types.BMesh.uv_select_flush_mode should be called afterwards.

SUBSTITUTION REPLACEMENT 'UV_SELECT_SYNC_TO_MESH_NEEDED':
This function doesn't flush the selection to the mesh, typically bmesh.types.BMesh.uv_select_sync_to_mesh should be called afterwards.


--------------------


--------------------


--------------------


--------------------


--------------------


--------------------

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types
import mathutils

class BMDeformVert:
    def clear(self) -> None:
        """Clears all weights."""

    def get(self, key: int, default: typing.Any | None = None) -> float | typing.Any:
        """Returns the deform weight matching the key or default
        when not found (matches Pythons dictionary function of the same name).

                :param key: The vertex group index.
                :param default: Optional argument for the value to return if
        key is not found.
                :return: The deform weight or the default when not found.
        """

    def items(self) -> list[tuple[int, float]]:
        """Return (group, weight) pairs for this vertex
        (matching Pythons dict.items() functionality).

                :return: (key, value) pairs for each deform weight of this vertex.
        """

    def keys(self) -> list[int]:
        """Return the group indices used by this vertex
        (matching Pythons dict.keys() functionality).

                :return: The deform group indices this vertex uses.
        """

    def values(self) -> list[float]:
        """Return the weights of the deform vertex
        (matching Pythons dict.values() functionality).

                :return: The weights that influence this vertex
        """

class BMEdge:
    """The BMesh edge connecting 2 verts"""

    hide: bool
    """ Hidden state of this element."""

    index: int
    """ Index of this element."""

    is_boundary: bool
    """ True when this edge is at the boundary of a face (read-only)."""

    is_contiguous: bool
    """ True when this edge is manifold, between two faces with the same winding (read-only)."""

    is_convex: bool
    """ True when this edge joins two convex faces, depends on a valid face normal (read-only)."""

    is_manifold: bool
    """ True when this edge is manifold (read-only)."""

    is_valid: bool
    """ True when this element is valid (hasn't been freed or removed)."""

    is_wire: bool
    """ True when this edge is not connected to any faces (read-only)."""

    link_faces: typing.Any
    """ Faces connected to this edge, (read-only)."""

    link_loops: typing.Any
    """ Loops connected to this edge, (read-only)."""

    seam: bool
    """ Seam for UV unwrapping."""

    select: bool
    """ Selected state of this element."""

    smooth: bool
    """ Smooth state of this element."""

    tag: bool
    """ Generic attribute scripts can use for own logic"""

    verts: typing.Any
    """ Verts this edge uses (always 2), (read-only)."""

    def calc_face_angle(self, fallback: typing.Any | None = None) -> float:
        """Return the angle between this edges two connected faces.

                :param fallback: return this when the edge doesnt have 2 faces
        (instead of raising a `ValueError`).
                :return: The angle between 2 connected faces in radians.
        """

    def calc_face_angle_signed(self, fallback: typing.Any | None = None) -> float:
        """Return the signed angle between this edges two connected faces.

                :param fallback: return this when the edge doesnt have 2 faces
        (instead of raising a `ValueError`).
                :return: The angle between 2 connected faces in radians (negative for concave join).
        """

    def calc_length(self) -> float:
        """Return the length of the edge.

        :return: The length between both verts.
        """

    def calc_tangent(self, loop: BMLoop) -> mathutils.Vector:
        """Return the tangent at this edge relative to a face (pointing inward into the face).
        This uses the face normal for calculation.

                :param loop: The loop used for tangent calculation.
                :return: a normalized vector.
        """

    def copy_from(self, other: typing_extensions.Self) -> None:
        """Copy values from another element of matching type.

        :param other: Another element of the same type to copy from.
        """

    def hide_set(self, hide: bool) -> None:
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
        """

    def normal_update(self) -> None:
        """Update normals of all connected faces and the edge verts."""

    def other_vert(self, vert: BMVert) -> BMVert | None:
        """Return the other vertex on this edge or None if the vertex is not used by this edge.

        :param vert: a vert in this edge.
        :return: The edges other vert.
        """

    def select_set(self, select: bool) -> None:
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
        """

    def __getitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1]
    ) -> _GenericType1:
        """

        :param key:
        :return:
        """

    def __setitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1], value: _GenericType1
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __delitem__[_GenericType1](self, key: BMLayerItem[_GenericType1]) -> None:
        """

        :param key:
        """

class BMEdgeSeq:
    layers: BMLayerAccessEdge
    """ custom-data layers (read-only)."""

    def ensure_lookup_table(self) -> None:
        """Ensure internal data needed for int subscript access is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""

    def get(
        self,
        verts: collections.abc.Sequence[BMVert],
        fallback: typing.Any | None = None,
    ) -> BMEdge | None:
        """Return an edge which uses the verts passed.

        :param verts: Pair of verts (exactly 2).
        :param fallback: Return this value if nothing is found.
        :return: The edge found or the fallback value.
        """

    def index_update(self) -> None:
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""

    def new(
        self,
        verts: collections.abc.Sequence[BMVert],
        source: BMEdge | None | None = None,
    ) -> BMEdge:
        """Create a new edge from a given pair of verts.

        :param verts: Vertex pair.
        :param source: Existing edge to initialize settings (optional argument).
        :return: The newly created edge.
        """

    def remove(self, edge: BMEdge) -> None:
        """Remove an edge.

        :param edge: The edge to remove.
        """

    def sort(
        self,
        *,
        key: None
        | collections.abc.Callable[[BMVert | BMEdge | BMFace], int]
        | None = None,
        reverse: bool = False,
    ) -> None:
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `bmesh.types.BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
        """

    @typing.overload
    def __getitem__(self, key: int) -> BMEdge:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> list[BMEdge]:
        """

        :param key:
        :return:
        """

    def __iter__(self) -> BMIter[BMEdge]:
        """

        :return:
        """

    def __len__(self) -> int:
        """

        :return:
        """

class BMEditSelIter: ...

class BMEditSelSeq:
    active: BMEdge | BMFace | BMVert | None
    """ The last selected element or None (read-only)."""

    def add(self, element: BMEdge | BMFace | BMVert) -> None:
        """Add an element to the selection history (no action taken if its already added).

        :param element: The element to add.
        """

    def clear(self) -> None:
        """Empties the selection history."""

    def discard(self, element: BMEdge | BMFace | BMVert) -> None:
        """Discard an element from the selection history.Like remove but doesnt raise an error when the element is not in the selection list.

        :param element: The element to discard.
        """

    def remove(self, element: BMEdge | BMFace | BMVert) -> None:
        """Remove an element from the selection history.

        :param element: The element to remove.
        """

    def validate(self) -> None:
        """Ensures all elements in the selection history are selected."""

class BMElemSeq[_GenericType1]:
    """General sequence type used for accessing any sequence of
    `bmesh.types.BMVert`, `bmesh.types.BMEdge`, `bmesh.types.BMFace`, `bmesh.types.BMLoop`.When accessed via `bmesh.types.BMesh.verts`, `bmesh.types.BMesh.edges`, `bmesh.types.BMesh.faces`
    there are also functions to create/remove items.
    """

    def index_update(self) -> None:
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""

    @typing.overload
    def __getitem__(self, key: int) -> _GenericType1:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> list[_GenericType1]:
        """

        :param key:
        :return:
        """

    def __iter__(self) -> BMIter[_GenericType1]:
        """

        :return:
        """

    def __len__(self) -> int:
        """

        :return:
        """

class BMFace:
    """The BMesh face with 3 or more sides"""

    edges: typing.Any
    """ Edges of this face, (read-only)."""

    hide: bool
    """ Hidden state of this element."""

    index: int
    """ Index of this element."""

    is_valid: bool
    """ True when this element is valid (hasn't been freed or removed)."""

    loops: typing.Any
    """ Loops of this face, (read-only)."""

    material_index: int
    """ The face's material index."""

    normal: mathutils.Vector
    """ The normal for this face as a 3D, wrapped vector."""

    select: bool
    """ Selected state of this element."""

    smooth: bool
    """ Smooth state of this element."""

    tag: bool
    """ Generic attribute scripts can use for own logic"""

    uv_select: bool
    """ UV selected state of this element."""

    verts: typing.Any
    """ Verts of this face, (read-only)."""

    def calc_area(self) -> float:
        """Return the area of the face.

        :return: The area of the face.
        """

    def calc_center_bounds(self) -> mathutils.Vector:
        """Return bounds center of the face.

        :return: a 3D vector.
        """

    def calc_center_median(self) -> mathutils.Vector:
        """Return median center of the face.

        :return: a 3D vector.
        """

    def calc_center_median_weighted(self) -> mathutils.Vector:
        """Return median center of the face weighted by edge lengths.

        :return: a 3D vector.
        """

    def calc_perimeter(self) -> float:
        """Return the perimeter of the face.

        :return: The perimeter of the face.
        """

    def calc_tangent_edge(self) -> mathutils.Vector:
        """Return face tangent based on longest edge.

        :return: a normalized vector.
        """

    def calc_tangent_edge_diagonal(self) -> mathutils.Vector:
        """Return face tangent based on the edge farthest from any vertex.

        :return: a normalized vector.
        """

    def calc_tangent_edge_pair(self) -> mathutils.Vector:
        """Return face tangent based on the two longest disconnected edges.

        :return: a normalized vector.
        """

    def calc_tangent_vert_diagonal(self) -> mathutils.Vector:
        """Return face tangent based on the two most distant vertices.

        :return: a normalized vector.
        """

    def copy(self, *, verts: bool = True, edges: bool = True) -> typing_extensions.Self:
        """Make a copy of this face.

        :param verts: When set, the faces verts will be duplicated too.
        :param edges: When set, the faces edges will be duplicated too.
        :return: The newly created face.
        """

    def copy_from(self, other: typing_extensions.Self) -> None:
        """Copy values from another element of matching type.

        :param other: Another element of the same type to copy from.
        """

    def copy_from_face_interp(
        self, face: typing_extensions.Self, vert: bool = True
    ) -> None:
        """Interpolate the customdata from another face onto this one (faces should overlap).

        :param face: The face to interpolate data from.
        :param vert: When True, also copy vertex data.
        """

    def hide_set(self, hide: bool) -> None:
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
        """

    def normal_flip(self) -> None:
        """Reverses winding of a face, which flips its normal."""

    def normal_update(self) -> None:
        """Update face normal based on the positions of the face verts.
        This does not update the normals of face verts.

        """

    def select_set(self, select: bool) -> None:
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
        """

    def uv_select_set(self, select: bool) -> None:
        """Set the UV face selection state.

        :param select: Select or de-select.
        """

    def __getitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1]
    ) -> _GenericType1:
        """

        :param key:
        :return:
        """

    def __setitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1], value: _GenericType1
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __delitem__[_GenericType1](self, key: BMLayerItem[_GenericType1]) -> None:
        """

        :param key:
        """

class BMFaceSeq:
    active: BMFace | None
    """ active face."""

    layers: BMLayerAccessFace
    """ custom-data layers (read-only)."""

    def ensure_lookup_table(self) -> None:
        """Ensure internal data needed for int subscript access is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""

    def get(
        self,
        verts: collections.abc.Sequence[BMVert],
        fallback: typing.Any | None = None,
    ) -> BMFace | None:
        """Return a face which uses the verts passed.

        :param verts: Sequence of verts.
        :param fallback: Return this value if nothing is found.
        :return: The face found or the fallback value.
        """

    def index_update(self) -> None:
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""

    def new(
        self,
        verts: collections.abc.Sequence[BMVert],
        source: BMFace | None | None = None,
    ) -> BMFace:
        """Create a new face from a given set of verts.

        :param verts: Sequence of 3 or more verts.
        :param source: Existing face to initialize settings (optional argument).
        :return: The newly created face.
        """

    def remove(self, face: BMFace) -> None:
        """Remove a face.

        :param face: The face to remove.
        """

    def sort(
        self,
        *,
        key: None
        | collections.abc.Callable[[BMVert | BMEdge | BMFace], int]
        | None = None,
        reverse: bool = False,
    ) -> None:
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `bmesh.types.BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
        """

    @typing.overload
    def __getitem__(self, key: int) -> BMFace:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> list[BMFace]:
        """

        :param key:
        :return:
        """

    def __iter__(self) -> BMIter[BMFace]:
        """

        :return:
        """

    def __len__(self) -> int:
        """

        :return:
        """

class BMIter[_GenericType1]:
    """Internal BMesh type for looping over verts/faces/edges,
    used for iterating over `bmesh.types.BMElemSeq` types.
    """

    def __iter__(self) -> BMIter[_GenericType1]:
        """

        :return:
        """

    def __next__(self) -> _GenericType1:
        """

        :return:
        """

class BMLayerAccessEdge:
    """Exposes custom-data layer attributes."""

    bool: typing.Any
    """ Generic boolean custom-data layer."""

    color: typing.Any
    """ Generic RGBA color with 8-bit precision custom-data layer."""

    float: typing.Any
    """ Generic float custom-data layer."""

    float_color: typing.Any
    """ Generic RGBA color with float precision custom-data layer."""

    float_vector: typing.Any
    """ Generic 3D vector with float precision custom-data layer."""

    int: typing.Any
    """ Generic int custom-data layer."""

    string: typing.Any
    """ Generic string custom-data layer (exposed as bytes, 255 max length)."""

class BMLayerAccessFace:
    """Exposes custom-data layer attributes."""

    bool: typing.Any
    """ Generic boolean custom-data layer."""

    color: typing.Any
    """ Generic RGBA color with 8-bit precision custom-data layer."""

    float: typing.Any
    """ Generic float custom-data layer."""

    float_color: typing.Any
    """ Generic RGBA color with float precision custom-data layer."""

    float_vector: typing.Any
    """ Generic 3D vector with float precision custom-data layer."""

    int: typing.Any
    """ Generic int custom-data layer."""

    string: typing.Any
    """ Generic string custom-data layer (exposed as bytes, 255 max length)."""

class BMLayerAccessLoop:
    """Exposes custom-data layer attributes."""

    bool: typing.Any
    """ Generic boolean custom-data layer."""

    color: typing.Any
    """ Generic RGBA color with 8-bit precision custom-data layer."""

    float: typing.Any
    """ Generic float custom-data layer."""

    float_color: typing.Any
    """ Generic RGBA color with float precision custom-data layer."""

    float_vector: typing.Any
    """ Generic 3D vector with float precision custom-data layer."""

    int: typing.Any
    """ Generic int custom-data layer."""

    string: typing.Any
    """ Generic string custom-data layer (exposed as bytes, 255 max length)."""

    uv: typing.Any
    """ Accessor for `bmesh.types.BMLoopUV` UV (as a 2D Vector)."""

class BMLayerAccessVert:
    """Exposes custom-data layer attributes."""

    bool: typing.Any
    """ Generic boolean custom-data layer."""

    color: typing.Any
    """ Generic RGBA color with 8-bit precision custom-data layer."""

    deform: typing.Any
    """ Vertex deform weight `bmesh.types.BMDeformVert` (TODO)."""

    float: typing.Any
    """ Generic float custom-data layer."""

    float_color: typing.Any
    """ Generic RGBA color with float precision custom-data layer."""

    float_vector: typing.Any
    """ Generic 3D vector with float precision custom-data layer."""

    int: typing.Any
    """ Generic int custom-data layer."""

    shape: typing.Any
    """ Vertex shape-key absolute location (as a 3D Vector)."""

    skin: typing.Any
    """ Accessor for skin layer."""

    string: typing.Any
    """ Generic string custom-data layer (exposed as bytes, 255 max length)."""

class BMLayerCollection[_GenericType1]:
    """Gives access to a collection of custom-data layers of the same type and behaves like Python dictionaries, except for the ability to do list like index access."""

    active: BMLayerItem[_GenericType1]
    is_singleton: bool
    """ True if there can exist only one layer of this type (read-only)."""

    def get[_GenericType2](
        self, key: str, default: _GenericType2 = None
    ) -> BMLayerItem | typing.Any:
        """Returns the value of the layer matching the key or default
        when not found (matches Pythons dictionary function of the same name).

                :param key: The key associated with the layer.
                :param default: Optional argument for the value to return if
        key is not found.
                :return: The layer matching the key or the default value.
        """

    def items(self) -> list[str, BMLayerItem[_GenericType1]]:
        """Return the (key, value) pairs of collection members
        (matching Pythons dict.items() functionality).

                :return: (key, value) pairs for each member of this collection.
        """

    def keys(self) -> list[str]:
        """Return the identifiers of collection members
        (matching Pythons dict.keys() functionality).

                :return: the identifiers for each member of this collection.
        """

    def new(self, name: str = "") -> BMLayerItem[_GenericType1]:
        """Create a new layer

        :param name: Optional name argument (will be made unique).
        :return: The newly created layer.
        """

    def remove(self, layer: BMLayerItem[_GenericType1]) -> None:
        """Remove a layer

        :param layer: The layer to remove.
        """

    def values(self) -> list[BMLayerItem[_GenericType1]]:
        """Return the values of collection
        (matching Pythons dict.values() functionality).

                :return: the members of this collection.
        """

    def verify(self) -> BMLayerItem[_GenericType1]:
        """Create a new layer or return an existing active layer

        :return: The newly created layer, or the existing active layer.
        """

class BMLayerItem[_GenericType1]:
    """Exposes a single custom data layer, its main purpose is for use as an item accessor to custom-data when used with vert/edge/face/loop data."""

    name: str
    """ The layer's unique name (read-only)."""

    def copy_from(self, other: typing_extensions.Self) -> None:
        """Copy data from another layer.

        :param other: Another layer to copy from.
        """

class BMLoop:
    """This is normally accessed from `bmesh.types.BMFace.loops` where each face loop represents a corner of the face."""

    edge: BMEdge
    """ The loop's edge (between this loop and the next), (read-only)."""

    face: BMFace
    """ The face this loop belongs to (read-only)."""

    index: int
    """ Index of this element."""

    is_convex: bool
    """ True when this loop is at the convex corner of a face, depends on a valid face normal (read-only)."""

    is_valid: bool
    """ True when this element is valid (hasn't been freed or removed)."""

    link_loop_next: typing_extensions.Self
    """ The next face corner (read-only)."""

    link_loop_prev: typing_extensions.Self
    """ The previous face corner (read-only)."""

    link_loop_radial_next: typing_extensions.Self
    """ The next loop around the edge (read-only)."""

    link_loop_radial_prev: typing_extensions.Self
    """ The previous loop around the edge (read-only)."""

    link_loops: typing.Any
    """ Loops connected to this loop, (read-only)."""

    tag: bool
    """ Generic attribute scripts can use for own logic"""

    uv_select_edge: bool
    """ UV edge selected state of this loop."""

    uv_select_vert: bool
    """ UV vertex selected state of this loop."""

    vert: BMVert
    """ The loop's vertex (read-only)."""

    def calc_angle(self) -> float:
        """Return the angle at this loops corner of the face.
        This is calculated so sharper corners give lower angles.

                :return: The angle in radians.
        """

    def calc_normal(self) -> mathutils.Vector:
        """Return normal at this loops corner of the face.
        Falls back to the face normal for straight lines.

                :return: a normalized vector.
        """

    def calc_tangent(self) -> mathutils.Vector:
        """Return the tangent at this loops corner of the face (pointing inward into the face).
        Falls back to the face normal for straight lines.

                :return: a normalized vector.
        """

    def copy_from(self, other: typing_extensions.Self) -> None:
        """Copy values from another element of matching type.

        :param other: Another element of the same type to copy from.
        """

    def copy_from_face_interp(
        self, face: BMFace, vert: bool = True, multires: bool = True
    ) -> None:
        """Interpolate the customdata from a face onto this loop (the loops vert should overlap the face).

        :param face: The face to interpolate data from.
        :param vert: When enabled, interpolate the loops vertex data (optional).
        :param multires: When enabled, interpolate the loops multires data (optional).
        """

    def uv_select_edge_set(self, select: bool) -> None:
        """Set the UV edge selection state.

        :param select: Select or de-select.
        """

    def uv_select_vert_set(self, select: bool) -> None:
        """Set the UV vertex selection state.

        :param select: Select or de-select.
        """

    def __getitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1]
    ) -> _GenericType1:
        """

        :param key:
        :return:
        """

    def __setitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1], value: _GenericType1
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __delitem__[_GenericType1](self, key: BMLayerItem[_GenericType1]) -> None:
        """

        :param key:
        """

class BMLoopSeq:
    layers: BMLayerAccessLoop
    """ custom-data layers (read-only)."""

    @typing.overload
    def __getitem__(self, key: int) -> BMLoop:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> list[BMLoop]:
        """

        :param key:
        :return:
        """

    def __iter__(self) -> BMIter[BMLoop]:
        """

        :return:
        """

    def __len__(self) -> int:
        """

        :return:
        """

class BMLoopUV:
    pin_uv: bool
    """ UV pin state."""

    uv: mathutils.Vector
    """ Loop UV (as a 2D Vector)."""

class BMVert:
    """The BMesh vertex type"""

    co: mathutils.Vector
    """ The coordinates for this vertex as a 3D, wrapped vector."""

    hide: bool
    """ Hidden state of this element."""

    index: int
    """ Index of this element."""

    is_boundary: bool
    """ True when this vertex is connected to boundary edges (read-only)."""

    is_manifold: bool
    """ True when this vertex is manifold (read-only)."""

    is_valid: bool
    """ True when this element is valid (hasn't been freed or removed)."""

    is_wire: bool
    """ True when this vertex is not connected to any faces (read-only)."""

    link_edges: typing.Any
    """ Edges connected to this vertex (read-only)."""

    link_faces: typing.Any
    """ Faces connected to this vertex (read-only)."""

    link_loops: typing.Any
    """ Loops that use this vertex (read-only)."""

    normal: mathutils.Vector
    """ The normal for this vertex as a 3D, wrapped vector."""

    select: bool
    """ Selected state of this element."""

    tag: bool
    """ Generic attribute scripts can use for own logic"""

    def calc_edge_angle(self, fallback: typing.Any | None = None) -> float:
        """Return the angle between this verts two connected edges.

                :param fallback: return this when the vert doesnt have 2 edges
        (instead of raising a `ValueError`).
                :return: Angle between edges in radians.
        """

    def calc_shell_factor(self) -> float:
        """Return a multiplier calculated based on the sharpness of the vertex.
        Where a flat surface gives 1.0, and higher values sharper edges.
        This is used to maintain shell thickness when offsetting verts along their normals.

                :return: offset multiplier
        """

    def copy_from(self, other: typing_extensions.Self) -> None:
        """Copy values from another element of matching type.

        :param other: Another element of the same type to copy from.
        """

    def copy_from_face_interp(self, face: BMFace) -> None:
        """Interpolate the customdata from a face onto this vert (the vert should overlap the face).

        :param face: The face to interpolate data from.
        """

    def copy_from_vert_interp(
        self, vert_pair: collections.abc.Sequence[BMVert], fac: float
    ) -> None:
        """Interpolate the customdata from a vert between 2 other verts.

        :param vert_pair: The verts between which to interpolate data from.
        :param fac: The interpolation factor.
        """

    def hide_set(self, hide: bool) -> None:
        """Set the hide state.
        This is different from the hide attribute because it updates the selection and hide state of associated geometry.

                :param hide: Hidden or visible.
        """

    def normal_update(self) -> None:
        """Update vertex normal.
        This does not update the normals of adjoining faces.

        """

    def select_set(self, select: bool) -> None:
        """Set the selection.
        This is different from the select attribute because it updates the selection state of associated geometry.

                :param select: Select or de-select.
        """

    def __getitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1]
    ) -> _GenericType1:
        """

        :param key:
        :return:
        """

    def __setitem__[_GenericType1](
        self, key: BMLayerItem[_GenericType1], value: _GenericType1
    ) -> None:
        """

        :param key:
        :param value:
        """

    def __delitem__[_GenericType1](self, key: BMLayerItem[_GenericType1]) -> None:
        """

        :param key:
        """

class BMVertSeq:
    layers: BMLayerAccessVert
    """ custom-data layers (read-only)."""

    def ensure_lookup_table(self) -> None:
        """Ensure internal data needed for int subscript access is initialized with verts/edges/faces, eg bm.verts[index].This needs to be called again after adding/removing data in this sequence."""

    def index_update(self) -> None:
        """Initialize the index values of this sequence.This is the equivalent of looping over all elements and assigning the index values."""

    def new(
        self,
        co: collections.abc.Sequence[float] | tuple[float, float, float] = (
            0.0,
            0.0,
            0.0,
        ),
        source: BMVert | None | None = None,
    ) -> BMVert:
        """Create a new vertex.

        :param co: The initial location of the vertex (optional argument).
        :param source: Existing vert to initialize settings.
        :return: The newly created vertex.
        """

    def remove(self, vert: BMVert) -> None:
        """Remove a vert.

        :param vert: The vert to remove.
        """

    def sort(
        self,
        *,
        key: None
        | collections.abc.Callable[[BMVert | BMEdge | BMFace], int]
        | None = None,
        reverse: bool = False,
    ) -> None:
        """Sort the elements of this sequence, using an optional custom sort key.
        Indices of elements are not changed, `bmesh.types.BMElemSeq.index_update` can be used for that.

                :param key: The key that sets the ordering of the elements.
                :param reverse: Reverse the order of the elements
        """

    @typing.overload
    def __getitem__(self, key: int) -> BMVert:
        """

        :param key:
        :return:
        """

    @typing.overload
    def __getitem__(self, key: slice) -> list[BMVert]:
        """

        :param key:
        :return:
        """

    def __iter__(self) -> BMIter[BMVert]:
        """

        :return:
        """

    def __len__(self) -> int:
        """

        :return:
        """

class BMesh:
    """The BMesh data structure"""

    edges: BMEdgeSeq
    """ This mesh's edge sequence (read-only)."""

    faces: BMFaceSeq
    """ This mesh's face sequence (read-only)."""

    is_valid: bool
    """ True when this element is valid (hasn't been freed or removed)."""

    is_wrapped: bool
    """ True when this mesh is owned by blender (typically the editmode BMesh)."""

    loops: BMLoopSeq
    """ This mesh's loops (read-only)."""

    select_history: BMEditSelSeq
    """ Sequence of selected items (the last is displayed as active)."""

    select_mode: set[typing.Literal["VERT", "EDGE", "FACE"]]
    """ The selection mode, cannot be assigned an empty set."""

    uv_select_sync_valid: bool
    """ When true, the UV selection has been synchronized. Setting to False means the UV selection will be ignored. While setting to true is supported it is up to the script author to ensure a correct selection state before doing so."""

    verts: BMVertSeq
    """ This mesh's vert sequence (read-only)."""

    def calc_loop_triangles(self) -> list[tuple[BMLoop, BMLoop, BMLoop]]:
        """Calculate triangle tessellation from quads/ngons.

        :return: The triangulated faces.
        """

    def calc_volume(self, *, signed: bool = False) -> float:
        """Calculate mesh volume based on face normals.

        :param signed: when signed is true, negative values may be returned.
        :return: The volume of the mesh.
        """

    def clear(self) -> None:
        """Clear all mesh data."""

    def copy(self) -> typing_extensions.Self:
        """

        :return: A copy of this BMesh.
        """

    def free(self) -> None:
        """Explicitly free the BMesh data from memory, causing exceptions on further access."""

    def from_mesh(
        self,
        mesh: bpy.types.Mesh,
        *,
        face_normals: bool = True,
        vertex_normals: bool = True,
        use_shape_key: bool = False,
        shape_key_index: int = 0,
    ) -> None:
        """Initialize this bmesh from existing mesh data-block.

        :param mesh: The mesh data to load.
        :param face_normals: Calculate face normals.
        :param vertex_normals: Calculate vertex normals.
        :param use_shape_key: Use the locations from a shape key.
        :param shape_key_index: The shape key index to use.
        """

    def from_object(
        self,
        object: bpy.types.Object,
        depsgraph: bpy.types.Depsgraph,
        *,
        cage: bool = False,
        face_normals: bool = True,
        vertex_normals: bool = True,
    ) -> None:
        """Initialize this bmesh from existing object data-block (only meshes are currently supported).

        :param object: The object data to load.
        :param depsgraph: The dependency graph for evaluated data.
        :param cage: Get the mesh as a deformed cage.
        :param face_normals: Calculate face normals.
        :param vertex_normals: Calculate vertex normals.
        """

    def normal_update(self) -> None:
        """Update normals of mesh faces and verts."""

    def select_flush(self, select: bool) -> None:
        """Flush selection from vertices, independent of the current selection mode.

        :param select: flush selection or de-selected elements.
        """

    def select_flush_mode(self, *, flush_down: bool = False) -> None:
        """Flush selection based on the current mode `bmesh.types.BMesh.select_mode`.

        :param flush_down: Flush selection down from faces to edges & verts or from edges to verts. This option is ignored when vertex selection mode is enabled.
        """

    def to_mesh(self, mesh: bpy.types.Mesh) -> None:
        """Writes this BMesh data into an existing Mesh data-block.

        :param mesh: The mesh data to write into.
        """

    def transform(
        self,
        matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
        | mathutils.Matrix,
        *,
        filter: None
        | set[typing.Literal["SELECT", "HIDE", "SEAM", "SMOOTH", "TAG"]]
        | None = None,
    ) -> None:
        """Transform the mesh (optionally filtering flagged data only).

        :param matrix: 4x4 transform matrix.
        :param filter: Flag to filter vertices.
        """

    def uv_select_flush(self, select: bool) -> None:
        """Flush selection from UV vertices to edges & faces independent of the selection mode.

        :param select: Flush selection or de-selected elements.
        """

    def uv_select_flush_mode(self, *, flush_down: bool = False) -> None:
        """Flush UV selection based on the current mode `bmesh.types.BMesh.select_mode`.

        :param flush_down: Flush selection down from faces to edges & verts or from edges to verts. This option is ignored when vertex selection mode is enabled.
        """

    def uv_select_flush_shared(self, select: bool) -> None:
        """Flush selection from UV vertices to contiguous UVs independent of the selection mode.

        :param select: Flush selection or de-selected elements.
        """

    def uv_select_foreach_set(
        self,
        select: bool,
        /,
        *,
        loop_verts: collections.abc.Iterable[BMLoop] = (),
        loop_edges: collections.abc.Iterable[BMLoop] = (),
        faces: collections.abc.Iterable[BMFace] = (),
        sticky_select_mode: typing.Literal[
            "SHARED_LOCATION", "DISABLED", "SHARED_VERTEX"
        ] = "SHARED_LOCATION",
    ) -> None:
        """Set the UV selection state for loop-vertices, loop-edges & faces.This is a close equivalent to selecting in the UV editor.

        :param select: The selection state to set.
        :param loop_verts: Loop verts to operate on.
        :param loop_edges: Loop edges to operate on.
        :param faces: Faces to operate on.
        :param sticky_select_mode: See UV_STICKY_SELECT_MODE_REF.
        """

    def uv_select_foreach_set_from_mesh(
        self,
        select: bool,
        /,
        *,
        verts: collections.abc.Iterable[BMVert] = (),
        edges: collections.abc.Iterable[BMEdge] = (),
        faces: collections.abc.Iterable[BMFace] = (),
        sticky_select_mode: typing.Literal[
            "SHARED_LOCATION", "DISABLED", "SHARED_VERTEX"
        ] = "SHARED_LOCATION",
    ) -> None:
        """Select or de-select mesh elements, updating the UV selection.An equivalent to selecting from the 3D viewport for selection operations that support maintaining a synchronized UV selection.

        :param select: The selection state to set.
        :param verts: Verts to operate on.
        :param edges: Edges to operate on.
        :param faces: Faces to operate on.
        :param sticky_select_mode: See UV_STICKY_SELECT_MODE_REF.
        """

    def uv_select_sync_from_mesh(
        self,
        *,
        sticky_select_mode: typing.Literal[
            "SHARED_LOCATION", "DISABLED", "SHARED_VERTEX"
        ] = "SHARED_LOCATION",
    ) -> None:
        """Sync selection from mesh to UVs.

        :param sticky_select_mode: Behavior when flushing from the mesh to UV selection UV_STICKY_SELECT_MODE_REF. This should only be used when preparing to create a UV selection.
        """

    def uv_select_sync_to_mesh(self) -> None:
        """Sync selection from UVs to the mesh."""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import gpu.types

def batch_for_shader(
    shader: gpu.types.GPUShader,
    type: typing.Literal[
        "POINTS",
        "LINES",
        "TRIS",
        "LINE_STRIP",
        "TRI_STRIP",
        "LINES_ADJ",
        "TRIS_ADJ",
        "LINE_STRIP_ADJ",
    ],
    content: dict[
        str,
        gpu.types.Buffer
        | collections.abc.Sequence[float]
        | collections.abc.Sequence[int]
        | collections.abc.Sequence[collections.abc.Sequence[float]]
        | collections.abc.Sequence[collections.abc.Sequence[int]],
    ],
    *,
    indices=None,
) -> gpu.types.GPUBatch:
    """Return a batch already configured and compatible with the shader.

        :param shader: shader for which a compatible format will be computed.
        :param type: The primitive type of batch geometry.
        :param content: Maps the name of the shader attribute with the data to fill the vertex buffer.
    For the dictionary values see documentation for `gpu.types.GPUVertBuf.attr_fill` data argument.
        :return: compatible batch
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bgui.widget

class Image(bgui.widget.Widget):
    """Widget for displaying images"""

    children: typing.Any
    image_size: typing.Any
    on_active: typing.Any
    on_click: typing.Any
    on_hover: typing.Any
    on_mouse_enter: typing.Any
    on_mouse_exit: typing.Any
    on_release: typing.Any
    parent: typing.Any
    position: typing.Any
    size: typing.Any
    system: typing.Any
    theme_options: typing.Any
    theme_section: typing.Any

    def update_image(self, img) -> None:
        """Changes the image texture

        :param img: the path to the new image
        :return:
        """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def autocomplete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Evaluate the namespace up until the cursor and give a list of options or complete the name if there is only one

    :return: Result of the operator call.
    """

def banner(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Print a message when the terminal initializes

    :return: Result of the operator call.
    """

def clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    scrollback: bool | None = True,
    history: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear text by type

    :param scrollback: Scrollback, Clear the scrollback history (optional)
    :param history: History, Clear the command history (optional)
    :return: Result of the operator call.
    """

def clear_line(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear the line and store in history

    :return: Result of the operator call.
    """

def copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    delete: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy selected text to clipboard

    :param delete: Delete Selection, Whether to delete the selection after copying (optional)
    :return: Result of the operator call.
    """

def copy_as_script(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy the console contents for use in a script

    :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "NEXT_CHARACTER", "PREVIOUS_CHARACTER", "NEXT_WORD", "PREVIOUS_WORD"
    ]
    | None = "NEXT_CHARACTER",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete text by cursor position

    :param type: Type, Which part of the text to delete (optional)
    :return: Result of the operator call.
    """

def execute(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    interactive: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Execute the current console line as a Python expression

    :param interactive: interactive, (optional)
    :return: Result of the operator call.
    """

def history_append(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    text: str | None = "",
    current_character: int | None = 0,
    remove_duplicates: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Append history at cursor position

    :param text: Text, Text to insert at the cursor position (optional, never None)
    :param current_character: Cursor, The index of the cursor (in [0, inf], optional)
    :param remove_duplicates: Remove Duplicates, Remove duplicate items in the history (optional)
    :return: Result of the operator call.
    """

def history_cycle(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    reverse: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Cycle through history

    :param reverse: Reverse, Reverse cycle history (optional)
    :return: Result of the operator call.
    """

def indent(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add 4 spaces at line beginning

    :return: Result of the operator call.
    """

def indent_or_autocomplete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Indent selected text or autocomplete

    :return: Result of the operator call.
    """

def insert(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    text: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Insert text at cursor position

    :param text: Text, Text to insert at the cursor position (optional, never None)
    :return: Result of the operator call.
    """

def language(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    language: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set the current language for this console

    :param language: Language, (optional, never None)
    :return: Result of the operator call.
    """

def move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "LINE_BEGIN",
        "LINE_END",
        "PREVIOUS_CHARACTER",
        "NEXT_CHARACTER",
        "PREVIOUS_WORD",
        "NEXT_WORD",
    ]
    | None = "LINE_BEGIN",
    select: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move cursor position

    :param type: Type, Where to move cursor to (optional)
    :param select: Select, Whether to select while moving (optional)
    :return: Result of the operator call.
    """

def paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    selection: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Paste text from clipboard

    :param selection: Selection, Paste text selected elsewhere rather than copied (X11/Wayland only) (optional)
    :return: Result of the operator call.
    """

def scrollback_append(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    text: str | None = "",
    type: typing.Literal["OUTPUT", "INPUT", "INFO", "ERROR"] | None = "OUTPUT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Append scrollback text by type

    :param text: Text, Text to insert at the cursor position (optional, never None)
    :param type: Type, Console output type (optional)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select all the text

    :return: Result of the operator call.
    """

def select_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set the console selection

    :return: Result of the operator call.
    """

def select_word(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select word at cursor position

    :return: Result of the operator call.
    """

def unindent(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete 4 spaces from line beginning

    :return: Result of the operator call.
    """



"""
The Blender geometry module.

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import mathutils

def area_tri(
    v1: collections.abc.Sequence[float] | mathutils.Vector,
    v2: collections.abc.Sequence[float] | mathutils.Vector,
    v3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> float:
    """Returns the area of the 2D or 3D triangle defined.

    :param v1: Point1
    :param v2: Point2
    :param v3: Point3
    :return: The area of the triangle.
    """

def barycentric_transform(
    point: collections.abc.Sequence[float] | mathutils.Vector,
    tri_a1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_a2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_a3: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> mathutils.Vector:
    """Return a transformed point, the transformation is defined by 2 triangles.

    :param point: The point to transform.
    :param tri_a1: source triangle vertex.
    :param tri_a2: source triangle vertex.
    :param tri_a3: source triangle vertex.
    :param tri_b1: target triangle vertex.
    :param tri_b2: target triangle vertex.
    :param tri_b3: target triangle vertex.
    :return: The transformed point
    """

def box_fit_2d(
    points: collections.abc.Sequence[collections.abc.Sequence[float]],
    /,
) -> float:
    """Returns an angle that best fits the points to an axis aligned rectangle

    :param points: Sequence of 2D points.
    :return: The rotation angle in radians for the best axis-aligned bounding box fit.
    """

def box_pack_2d(
    boxes: list[list[float]],
    /,
) -> tuple[float, float]:
    """Returns a tuple with the width and height of the packed bounding box.

    :param boxes: list of boxes, each box is a list where the first 4 items are [X, Y, width, height, ...] other items are ignored. The X & Y values in this list are modified to set the packed positions.
    :return: The width and height of the packed bounding box.
    """

def closest_point_on_tri(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> mathutils.Vector:
    """Takes 4 vectors: one is the point and the next 3 define the triangle.

    :param pt: Point
    :param tri_p1: First point of the triangle
    :param tri_p2: Second point of the triangle
    :param tri_p3: Third point of the triangle
    :return: The closest point of the triangle.
    """

def convex_hull_2d(
    points: collections.abc.Sequence[collections.abc.Sequence[float]],
    /,
) -> list[int]:
    """Returns the indices of the points forming the convex hull, in counter-clockwise order.

    :param points: Sequence of 2D points.
    :return: Indices of convex hull vertices in counter-clockwise order.
    """

def delaunay_2d_cdt(
    vert_coords: collections.abc.Sequence[
        collections.abc.Sequence[float] | mathutils.Vector
    ],
    edges: collections.abc.Sequence[tuple[int, int]],
    faces: collections.abc.Sequence[collections.abc.Sequence[int]],
    output_type: int,
    epsilon: float,
    need_ids: bool = True,
    /,
) -> tuple[
    list[mathutils.Vector],
    list[tuple[int, int]],
    list[list[int]],
    list[list[int]],
    list[list[int]],
    list[list[int]],
]:
    """Computes the Constrained Delaunay Triangulation of a set of vertices,
    with edges and faces that must appear in the triangulation.
    Some triangles may be eaten away, or combined with other triangles,
    according to output type.
    The returned verts may be in a different order from input verts, may be moved
    slightly, and may be merged with other nearby verts.
    The three returned orig lists give, for each of verts, edges, and faces, the list of
    input element indices corresponding to the positionally same output element.
    For edges, the orig indices start with the input edges and then continue
    with the edges implied by each of the faces (n of them for an n-gon).
    If the need_ids argument is supplied, and False, then the code skips the preparation
    of the orig arrays, which may save some time.

        :param vert_coords: Vertex coordinates (2d)
        :param edges: Edges, as pairs of indices in vert_coords
        :param faces: Faces, each sublist is a face, as indices in vert_coords (CCW oriented).
        :param output_type: What output looks like. 0 => triangles with convex hull. 1 => triangles inside constraints. 2 => the input constraints, intersected. 3 => like 2 but detect holes and omit them from output. 4 => like 2 but with extra edges to make valid BMesh faces. 5 => like 4 but detect holes and omit them from output.
        :param epsilon: For nearness tests; should not be zero
        :param need_ids: are the orig output arrays needed?
        :return: Output tuple, (vert_coords, edges, faces, orig_verts, orig_edges, orig_faces)
    """

def distance_point_to_plane(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    plane_co: collections.abc.Sequence[float] | mathutils.Vector,
    plane_no: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> float:
    """Returns the signed distance between a point and a plane (negative when below the normal).

    :param pt: Point
    :param plane_co: A point on the plane
    :param plane_no: The direction the plane is facing
    :return: The signed distance.
    """

def interpolate_bezier(
    knot1: collections.abc.Sequence[float] | mathutils.Vector,
    handle1: collections.abc.Sequence[float] | mathutils.Vector,
    handle2: collections.abc.Sequence[float] | mathutils.Vector,
    knot2: collections.abc.Sequence[float] | mathutils.Vector,
    resolution: int,
    /,
) -> list[mathutils.Vector]:
    """Interpolate a bezier spline segment.

    :param knot1: First bezier spline point.
    :param handle1: First bezier spline handle.
    :param handle2: Second bezier spline handle.
    :param knot2: Second bezier spline point.
    :param resolution: Number of points to return.
    :return: The interpolated points.
    """

def intersect_line_line(
    v1: collections.abc.Sequence[float] | mathutils.Vector,
    v2: collections.abc.Sequence[float] | mathutils.Vector,
    v3: collections.abc.Sequence[float] | mathutils.Vector,
    v4: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> None | tuple[mathutils.Vector, mathutils.Vector]:
    """Returns a tuple with the points on each line respectively closest to the other.

    :param v1: First point of the first line
    :param v2: Second point of the first line
    :param v3: First point of the second line
    :param v4: Second point of the second line
    :return: The intersection on each line or None when the lines are parallel.
    """

def intersect_line_line_2d(
    lineA_p1: collections.abc.Sequence[float] | mathutils.Vector,
    lineA_p2: collections.abc.Sequence[float] | mathutils.Vector,
    lineB_p1: collections.abc.Sequence[float] | mathutils.Vector,
    lineB_p2: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> None | mathutils.Vector:
    """Takes 2 segments (defined by 4 vectors) and returns a vector for their point of intersection or None.

    :param lineA_p1: First point of the first segment
    :param lineA_p2: Second point of the first segment
    :param lineB_p1: First point of the second segment
    :param lineB_p2: Second point of the second segment
    :return: The point of intersection or None when not found
    """

def intersect_line_plane(
    line_a: collections.abc.Sequence[float] | mathutils.Vector,
    line_b: collections.abc.Sequence[float] | mathutils.Vector,
    plane_co: collections.abc.Sequence[float] | mathutils.Vector,
    plane_no: collections.abc.Sequence[float] | mathutils.Vector,
    no_flip: bool = False,
    /,
) -> None | mathutils.Vector:
    """Calculate the intersection between a line (as 2 vectors) and a plane.
    Returns a vector for the intersection or None.

        :param line_a: First point of the line
        :param line_b: Second point of the line
        :param plane_co: A point on the plane
        :param plane_no: The direction the plane is facing
        :param no_flip: Currently ignored.
        :return: The point of intersection or None when not found
    """

def intersect_line_sphere(
    line_a: collections.abc.Sequence[float] | mathutils.Vector,
    line_b: collections.abc.Sequence[float] | mathutils.Vector,
    sphere_co: collections.abc.Sequence[float] | mathutils.Vector,
    sphere_radius: float,
    clip: bool = True,
    /,
) -> tuple[mathutils.Vector | None, mathutils.Vector | None]:
    """Takes a line (as 2 points) and a sphere (as a point and a radius) and
    returns the intersection

        :param line_a: First point of the line
        :param line_b: Second point of the line
        :param sphere_co: The center of the sphere
        :param sphere_radius: Radius of the sphere
        :param clip: When False, dont restrict the intersection to the line segment.
        :return: The intersection points as a pair of vectors (each is None when not found).
    """

def intersect_line_sphere_2d(
    line_a: collections.abc.Sequence[float] | mathutils.Vector,
    line_b: collections.abc.Sequence[float] | mathutils.Vector,
    sphere_co: collections.abc.Sequence[float] | mathutils.Vector,
    sphere_radius: float,
    clip: bool = True,
    /,
) -> tuple[mathutils.Vector | None, mathutils.Vector | None]:
    """Takes a line (as 2 points) and a circle (as a point and a radius) and
    returns the intersection

        :param line_a: First point of the line
        :param line_b: Second point of the line
        :param sphere_co: The center of the circle
        :param sphere_radius: Radius of the circle
        :param clip: When False, dont restrict the intersection to the line segment.
        :return: The intersection points as a pair of vectors (each is None when not found).
    """

def intersect_plane_plane(
    plane_a_co: collections.abc.Sequence[float] | mathutils.Vector,
    plane_a_no: collections.abc.Sequence[float] | mathutils.Vector,
    plane_b_co: collections.abc.Sequence[float] | mathutils.Vector,
    plane_b_no: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> tuple[None, None] | tuple[mathutils.Vector, mathutils.Vector]:
    """Return the intersection between two planes

    :param plane_a_co: Point on the first plane
    :param plane_a_no: Normal of the first plane
    :param plane_b_co: Point on the second plane
    :param plane_b_no: Normal of the second plane
    :return: The line of the intersection represented as a point and a vector or None if the intersection cant be calculated
    """

def intersect_point_line(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    line_p1: collections.abc.Sequence[float] | mathutils.Vector,
    line_p2: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> tuple[mathutils.Vector, float]:
    """Takes a point and a line and returns the closest point on the line and its parametric distance from the first point of the line. A value of 0.0 is the first point, 1.0 is the second, values outside [0, 1] are extrapolated.

    :param pt: Point
    :param line_p1: First point of the line
    :param line_p2: Second point of the line
    :return: The closest point on the line and its parametric distance from the first point.
    """

def intersect_point_line_segment(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    seg_p1: collections.abc.Sequence[float] | mathutils.Vector,
    seg_p2: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> tuple[mathutils.Vector, float]:
    """Takes a point and a segment and returns the closest point on the segment and the distance to the segment.

    :param pt: Point
    :param seg_p1: First point of the segment
    :param seg_p2: Second point of the segment
    :return: The closest point on the segment and the distance to the segment.
    """

def intersect_point_quad_2d(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    quad_p1: collections.abc.Sequence[float] | mathutils.Vector,
    quad_p2: collections.abc.Sequence[float] | mathutils.Vector,
    quad_p3: collections.abc.Sequence[float] | mathutils.Vector,
    quad_p4: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> int:
    """Takes 5 vectors (using only the x and y coordinates): one is the point and the next 4 define the quad,
    only the x and y are used from the vectors. Returns a non-zero value if the point is within the quad, otherwise 0.
    Works only with convex quads without singular edges.

        :param pt: Point
        :param quad_p1: First point of the quad
        :param quad_p2: Second point of the quad
        :param quad_p3: Third point of the quad
        :param quad_p4: Fourth point of the quad
        :return: 1 if inside with CCW winding, -1 if inside with CW winding, otherwise 0.
    """

def intersect_point_tri(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> None | mathutils.Vector:
    """Takes 4 vectors: one is the point and the next 3 define the triangle. Projects the point onto the triangle plane and checks if it is within the triangle.

    :param pt: Point
    :param tri_p1: First point of the triangle
    :param tri_p2: Second point of the triangle
    :param tri_p3: Third point of the triangle
    :return: Point on the triangles plane or None if its outside the triangle
    """

def intersect_point_tri_2d(
    pt: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_p3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> int:
    """Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns a non-zero value if the point is within the triangle, otherwise 0.

    :param pt: Point
    :param tri_p1: First point of the triangle
    :param tri_p2: Second point of the triangle
    :param tri_p3: Third point of the triangle
    :return: 1 if inside with CCW winding, -1 if inside with CW winding, otherwise 0.
    """

def intersect_ray_tri(
    v1: collections.abc.Sequence[float] | mathutils.Vector,
    v2: collections.abc.Sequence[float] | mathutils.Vector,
    v3: collections.abc.Sequence[float] | mathutils.Vector,
    ray: collections.abc.Sequence[float] | mathutils.Vector,
    orig: collections.abc.Sequence[float] | mathutils.Vector,
    clip: bool = True,
    /,
) -> None | mathutils.Vector:
    """Returns the intersection between a ray and a triangle, if possible, returns None otherwise.

    :param v1: Point1
    :param v2: Point2
    :param v3: Point3
    :param ray: Direction of the ray
    :param orig: Origin
    :param clip: When False, dont restrict the intersection to the area of the triangle, use the infinite plane defined by the triangle.
    :return: The point of intersection or None if no intersection is found
    """

def intersect_sphere_sphere_2d(
    p_a: collections.abc.Sequence[float] | mathutils.Vector,
    radius_a: float,
    p_b: collections.abc.Sequence[float] | mathutils.Vector,
    radius_b: float,
    /,
) -> tuple[None, None] | tuple[mathutils.Vector, mathutils.Vector]:
    """Returns the 2 intersection points of two circles.

    :param p_a: Center of the first circle
    :param radius_a: Radius of the first circle
    :param p_b: Center of the second circle
    :param radius_b: Radius of the second circle
    :return: The 2 intersection points or None when there is no intersection.
    """

def intersect_tri_tri_2d(
    tri_a1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_a2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_a3: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b1: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b2: collections.abc.Sequence[float] | mathutils.Vector,
    tri_b3: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> bool:
    """Check if two 2D triangles intersect.

    :param tri_a1: First vertex of the first triangle.
    :param tri_a2: Second vertex of the first triangle.
    :param tri_a3: Third vertex of the first triangle.
    :param tri_b1: First vertex of the second triangle.
    :param tri_b2: Second vertex of the second triangle.
    :param tri_b3: Third vertex of the second triangle.
    :return: True if the triangles intersect.
    """

def normal(
    *vectors: collections.abc.Sequence[collections.abc.Sequence[float]],
) -> mathutils.Vector:
    """Returns the normal of a 3D polygon.

    :param vectors: 3 or more vectors to calculate normals.
    :return: The normal vector.
    """

def points_in_planes(
    planes: list[collections.abc.Sequence[float] | mathutils.Vector],
    epsilon_coplanar: float = 0.0001,
    epsilon_isect: float = 1e-06,
    /,
) -> tuple[list[mathutils.Vector], list[int]]:
    """Returns a list of points inside all planes given and a list of index values for the planes used.

    :param planes: List of planes (4D vectors).
    :param epsilon_coplanar: Epsilon value for interpreting plane pairs as co-planar.
    :param epsilon_isect: Epsilon value for intersection.
    :return: Two lists, one containing the 3D coordinates inside the planes, another containing the plane indices used.
    """

def tessellate_polygon(
    polylines: collections.abc.Sequence[
        collections.abc.Sequence[collections.abc.Sequence[float]]
    ],
    /,
) -> list[tuple[int, int, int]]:
    """Takes a list of polylines (each point a pair or triplet of numbers) and returns the point indices for a polyline filled with triangles. Does not handle degenerate geometry (such as zero-length lines due to consecutive identical points).

    :param polylines: Polygons where each polygon is a sequence of 2D or 3D points.
    :return: A list of triangles.
    """

def volume_tetrahedron(
    v1: collections.abc.Sequence[float] | mathutils.Vector,
    v2: collections.abc.Sequence[float] | mathutils.Vector,
    v3: collections.abc.Sequence[float] | mathutils.Vector,
    v4: collections.abc.Sequence[float] | mathutils.Vector,
    /,
) -> float:
    """Return the absolute (unsigned) volume formed by a tetrahedron (points can be in any order).

    :param v1: Point1
    :param v2: Point2
    :param v3: Point3
    :param v4: Point4
    :return: The volume of the tetrahedron.
    """



"""
This module provides access to ID property types, used for
custom properties on data-blocks, accessed via ["key"]

 syntax.

* See info_quickstart-custom_properties for example usage.
* See bpy_types-custom_properties for types that support custom properties.

idprop.types.rst

:maxdepth: 1
:caption: Submodules

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
from . import types as types



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

def keyconfig_data_oskey_from_ctrl(keyconfig_data_src, *, filter_fn=None) -> None: ...
def keyconfig_data_oskey_from_ctrl_for_macos(keyconfig_data_src) -> None:
    """Use for apple since Cmd is typically used in-place of Ctrl."""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def addon_disable(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    module: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Turn off this add-on

    :param module: Module, Module name of the add-on to disable (optional, never None)
    :return: Result of the operator call.
    """

def addon_enable(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    module: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Turn on this add-on

    :param module: Module, Module name of the add-on to enable (optional, never None)
    :return: Result of the operator call.
    """

def addon_expand(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    module: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Display information and preferences for this add-on

    :param module: Module, Module name of the add-on to expand (optional, never None)
    :return: Result of the operator call.
    """

def addon_install(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    overwrite: bool | None = True,
    enable_on_install: bool | None = False,
    target: str | None = "",
    filepath: str | None = "",
    filter_folder: bool | None = True,
    filter_python: bool | None = True,
    filter_glob: str | None = "*.py;*.zip",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Install an add-on

    :param overwrite: Overwrite, Remove existing add-ons with the same ID (optional)
    :param enable_on_install: Enable on Install, Enable after installing (optional)
    :param target: Target Path, (optional)
    :param filepath: filepath, (optional, never None)
    :param filter_folder: Filter folders, (optional)
    :param filter_python: Filter Python, (optional)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def addon_refresh(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Scan add-on directories for new modules

    :return: Result of the operator call.
    """

def addon_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    module: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete the add-on from the file system

    :param module: Module, Module name of the add-on to remove (optional, never None)
    :return: Result of the operator call.
    """

def addon_show(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    module: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Show add-on preferences

    :param module: Module, Module name of the add-on to expand (optional, never None)
    :return: Result of the operator call.
    """

def app_template_install(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    overwrite: bool | None = True,
    filepath: str | None = "",
    filter_folder: bool | None = True,
    filter_glob: str | None = "*.zip",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Install an application template

    :param overwrite: Overwrite, Remove existing template with the same ID (optional)
    :param filepath: filepath, (optional, never None)
    :param filter_folder: Filter folders, (optional)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def asset_library_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    directory: str | None = "",
    hide_props_region: bool | None = True,
    check_existing: bool | None = False,
    filter_blender: bool | None = False,
    filter_backup: bool | None = False,
    filter_image: bool | None = False,
    filter_movie: bool | None = False,
    filter_python: bool | None = False,
    filter_font: bool | None = False,
    filter_sound: bool | None = False,
    filter_text: bool | None = False,
    filter_archive: bool | None = False,
    filter_btx: bool | None = False,
    filter_alembic: bool | None = False,
    filter_usd: bool | None = False,
    filter_obj: bool | None = False,
    filter_volume: bool | None = False,
    filter_folder: bool | None = True,
    filter_blenlib: bool | None = False,
    filemode: int | None = 9,
    display_type: typing.Literal[
        "DEFAULT", "LIST_VERTICAL", "LIST_HORIZONTAL", "THUMBNAIL"
    ]
    | None = "DEFAULT",
    sort_method: str | None = "",
    name: str | None = "",
    remote_url: str | None = "",
    type: typing.Literal["REMOTE", "LOCAL"] | None = "REMOTE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a directory to be used by the Asset Browser as source of assets

        :param directory: Directory, Directory of the file (optional, never None)
        :param hide_props_region: Hide Operator Properties, Collapse the region displaying the operator settings (optional)
        :param check_existing: Check Existing, Check and warn on overwriting existing files (optional)
        :param filter_blender: Filter .blend files, (optional)
        :param filter_backup: Filter backup .blend files, (optional)
        :param filter_image: Filter image files, (optional)
        :param filter_movie: Filter movie files, (optional)
        :param filter_python: Filter Python files, (optional)
        :param filter_font: Filter font files, (optional)
        :param filter_sound: Filter sound files, (optional)
        :param filter_text: Filter text files, (optional)
        :param filter_archive: Filter archive files, (optional)
        :param filter_btx: Filter btx files, (optional)
        :param filter_alembic: Filter Alembic files, (optional)
        :param filter_usd: Filter USD files, (optional)
        :param filter_obj: Filter OBJ files, (optional)
        :param filter_volume: Filter OpenVDB volume files, (optional)
        :param filter_folder: Filter folders, (optional)
        :param filter_blenlib: Filter Blender IDs, (optional)
        :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file (in [1, 9], optional)
        :param display_type: Display Type, (optional)

    DEFAULT
    Default -- Automatically determine display type for files.

    LIST_VERTICAL
    Short List -- Display files as short list.

    LIST_HORIZONTAL
    Long List -- Display files as a detailed list.

    THUMBNAIL
    Thumbnails -- Display files as thumbnails.
        :param sort_method: File sorting mode, (optional)
        :param name: Name, Identifier (not necessarily unique) for the asset library (optional, never None)
        :param remote_url: URL, Remote URL to the asset library (optional, never None)
        :param type: Type, The kind of asset library to add (optional)

    REMOTE
    Add Remote Asset Library -- Add an asset library referencing a remote repository with support for listing and updating asset libraries.

    LOCAL
    Add Local Asset Library -- Add an asset library managed via the file system without referencing an external repository.
        :return: Result of the operator call.
    """

def asset_library_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove a path to a .blend file, so the Asset Browser will not attempt to show it anymore

    :param index: Index, (in [0, inf], optional)
    :return: Result of the operator call.
    """

def associate_blend(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Use this installation for .blend files and to display thumbnails

    :return: Result of the operator call.
    """

def autoexec_path_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add path to exclude from auto-execution

    :return: Result of the operator call.
    """

def autoexec_path_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove path to exclude from auto-execution

    :param index: Index, (in [0, inf], optional)
    :return: Result of the operator call.
    """

def clear_filter(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear the search filter

    :return: Result of the operator call.
    """

def copy_prev(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy settings from previous version

    :return: Result of the operator call.
    """

def extension_repo_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    name: str | None = "",
    remote_url: str | None = "",
    use_access_token: bool | None = False,
    access_token: str | None = "",
    use_sync_on_startup: bool | None = False,
    use_custom_directory: bool | None = False,
    custom_directory: str | None = "",
    type: typing.Literal["REMOTE", "LOCAL"] | None = "REMOTE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a new repository used to store extensions

        :param name: Name, Unique repository name (optional, never None)
        :param remote_url: URL, Remote URL to the extension repository, the file-system may be referenced using the file URI scheme: "file://" (optional, never None)
        :param use_access_token: Requires Access Token, Repository requires an access token (optional)
        :param access_token: Secret, Personal access token, may be required by some repositories (optional, never None)
        :param use_sync_on_startup: Check for Updates on Startup, Allow Blender to check for updates upon launch (optional)
        :param use_custom_directory: Custom Directory, Manually set the path for extensions to be stored. When disabled a users extensions directory is created. (optional)
        :param custom_directory: Custom Directory, The local directory containing extensions (optional, never None)
        :param type: Type, The kind of repository to add (optional)

    REMOTE
    Add Remote Repository -- Add a repository referencing a remote repository with support for listing and updating extensions.

    LOCAL
    Add Local Repository -- Add a repository managed manually without referencing an external repository.
        :return: Result of the operator call.
    """

def extension_repo_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
    remove_files: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove an extension repository

    :param index: Index, (in [0, inf], optional)
    :param remove_files: Remove Files, Remove extension files when removing the repository (optional)
    :return: Result of the operator call.
    """

def extension_url_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    url: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Handle dropping an extension URL

    :param url: URL, Location of the extension to install (optional, never None)
    :return: Result of the operator call.
    """

def keyconfig_activate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param filepath: filepath, (optional, never None)
    :return: Result of the operator call.
    """

def keyconfig_export(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    all: bool | None = False,
    filepath: str | None = "",
    filter_folder: bool | None = True,
    filter_text: bool | None = True,
    filter_python: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Export key configuration to a Python script

    :param all: All Keymaps, Write all keymaps (not just user modified) (optional)
    :param filepath: filepath, (optional, never None)
    :param filter_folder: Filter folders, (optional)
    :param filter_text: Filter text, (optional)
    :param filter_python: Filter Python, (optional)
    :return: Result of the operator call.
    """

def keyconfig_import(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "keymap.py",
    filter_folder: bool | None = True,
    filter_text: bool | None = True,
    filter_python: bool | None = True,
    keep_original: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Import key configuration from a Python script

    :param filepath: filepath, (optional, never None)
    :param filter_folder: Filter folders, (optional)
    :param filter_text: Filter text, (optional)
    :param filter_python: Filter Python, (optional)
    :param keep_original: Keep Original, Keep original file after copying to configuration folder (optional)
    :return: Result of the operator call.
    """

def keyconfig_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove key config

    :return: Result of the operator call.
    """

def keyconfig_test(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Test key configuration for conflicts

    :return: Result of the operator call.
    """

def keyitem_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add key map item

    :return: Result of the operator call.
    """

def keyitem_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    item_id: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove key map item

    :param item_id: Item Identifier, Identifier of the item to remove (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def keyitem_restore(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    item_id: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Restore key map item

    :param item_id: Item Identifier, Identifier of the item to restore (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def keymap_restore(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    all: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Restore key map(s)

    :param all: All Keymaps, Restore all keymaps to default (optional)
    :return: Result of the operator call.
    """

def reset_default_theme(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset to the default theme colors

    :return: Result of the operator call.
    """

def script_directory_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    directory: str | None = "",
    filter_folder: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param directory: directory, (optional, never None)
    :param filter_folder: Filter Folders, (optional)
    :return: Result of the operator call.
    """

def script_directory_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param index: Index, Index of the script directory to remove (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def start_filter(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Start entering filter text

    :return: Result of the operator call.
    """

def studiolight_copy_settings(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy Studio Light settings to the Studio Light editor

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def studiolight_install(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    files=None,
    directory: str | None = "",
    filter_folder: bool | None = True,
    filter_glob: str | None = "*.png;*.jpg;*.hdr;*.exr",
    type: typing.Literal["MATCAP", "WORLD", "STUDIO"] | None = "MATCAP",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Install a user defined light

        :param files: File Path, (optional)
        :param directory: directory, (optional, never None)
        :param filter_folder: Filter Folders, (optional)
        :param filter_glob: filter_glob, (optional, never None)
        :param type: Type, (optional)

    MATCAP
    MatCap -- Install custom MatCaps.

    WORLD
    World -- Install custom HDRIs.

    STUDIO
    Studio -- Install custom Studio Lights.
        :return: Result of the operator call.
    """

def studiolight_new(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filename: str | None = "StudioLight",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Save custom studio light from the studio light editor settings

    :param filename: Name, (optional, never None)
    :return: Result of the operator call.
    """

def studiolight_uninstall(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete Studio Light

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def theme_install(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    overwrite: bool | None = True,
    filepath: str | None = "",
    filter_folder: bool | None = True,
    filter_glob: str | None = "*.xml",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Load and apply a Blender XML theme file

    :param overwrite: Overwrite, Remove existing theme file if exists (optional)
    :param filepath: filepath, (optional, never None)
    :param filter_folder: Filter folders, (optional)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def unassociate_blend(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove this installations associations with .blend files

    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def attribute_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    value_float: float | None = 0.0,
    value_float_vector_2d: collections.abc.Sequence[float] | None = (0.0, 0.0),
    value_float_vector_3d: collections.abc.Sequence[float] | None = (0.0, 0.0, 0.0),
    value_float_vector_4d: collections.abc.Sequence[float] | None = (
        0.0,
        0.0,
        0.0,
        0.0,
    ),
    value_int: int | None = 0,
    value_int_vector_2d: collections.abc.Sequence[int] | None = (0, 0),
    value_color: collections.abc.Sequence[float] | None = (1.0, 1.0, 1.0, 1.0),
    value_bool: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set values of the active attribute for selected elements

    :param value_float: Value, (in [-inf, inf], optional)
    :param value_float_vector_2d: Value, (array of 2 items, in [-inf, inf], optional)
    :param value_float_vector_3d: Value, (array of 3 items, in [-inf, inf], optional)
    :param value_float_vector_4d: Value, (array of 4 items, in [-inf, inf], optional)
    :param value_int: Value, (in [-inf, inf], optional)
    :param value_int_vector_2d: Value, (array of 2 items, in [-inf, inf], optional)
    :param value_color: Value, (array of 4 items, in [-inf, inf], optional)
    :param value_bool: Value, (optional)
    :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove selected points

    :return: Result of the operator call.
    """

def duplicate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy selected points

    :return: Result of the operator call.
    """

def duplicate_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    POINTCLOUD_OT_duplicate: dict[str, typing.Any] | None = {},
    TRANSFORM_OT_translate: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Make copies of selected elements and move them

    :param POINTCLOUD_OT_duplicate: Duplicate, Copy selected points (optional, `bpy.ops.pointcloud.duplicate` keyword arguments)
    :param TRANSFORM_OT_translate: Move, Move selected items (optional, `bpy.ops.transform.translate` keyword arguments)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """(De)select all points

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_random(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    seed: int | None = 0,
    probability: float | None = 0.5,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Randomize existing selection or create new random selection

    :param seed: Seed, Source of randomness (in [-inf, inf], optional)
    :param probability: Probability, Chance of every point being included in the selection (in [0, 1], optional)
    :return: Result of the operator call.
    """

def separate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Separate selected geometry into a new point cloud

    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def flip(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    axis: typing.Literal["U", "V", "W"] | None = "U",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Mirror all control points without inverting the lattice deform

    :param axis: Flip Axis, Coordinates along this axis get flipped (optional)
    :return: Result of the operator call.
    """

def make_regular(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set UVW control points a uniform distance apart

    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Change selection of all UVW control points

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_less(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Deselect vertices at the boundary of each selection region

    :return: Result of the operator call.
    """

def select_mirror(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    axis: set[Literal[bpy.stub_internal.rna_enums.AxisFlagXyzItems]] | None = {"X"},
    extend: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select mirrored lattice points

    :param axis: Axis, (optional)
    :param extend: Extend, Extend the selection (optional)
    :return: Result of the operator call.
    """

def select_more(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select vertices directly linked to already selected ones

    :return: Result of the operator call.
    """

def select_random(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    ratio: float | None = 0.5,
    seed: int | None = 0,
    action: typing.Literal["SELECT", "DESELECT"] | None = "SELECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Randomly select UVW control points

        :param ratio: Ratio, Portion of items to select randomly (in [0, 1], optional)
        :param seed: Random Seed, Seed for the random number generator (in [0, inf], optional)
        :param action: Action, Selection action to execute (optional)

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.
        :return: Result of the operator call.
    """

def select_ungrouped(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    extend: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select vertices without a group

    :param extend: Extend, Extend the selection (optional)
    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types

class ToolActivePanelHelper:
    bl_label: typing.Any

    def draw(self, context) -> None:
        """

        :param context:
        """

class ToolDef:
    brush_type: typing.Any
    cursor: typing.Any
    data_block: typing.Any
    description: typing.Any
    draw_cursor: typing.Any
    draw_settings: typing.Any
    icon: typing.Any
    idname: typing.Any
    keymap: typing.Any
    label: typing.Any
    operator: typing.Any
    options: typing.Any
    widget: typing.Any
    widget_properties: typing.Any

class ToolSelectPanelHelper:
    def draw(self, context) -> None:
        """

        :param context:
        """

    @staticmethod
    def draw_active_tool_fallback(
        context, layout, tool, *, is_horizontal_layout=False
    ) -> None:
        """

        :param context:
        :param layout:
        :param tool:
        :param is_horizontal_layout:
        """

    @staticmethod
    def draw_active_tool_header(
        context, layout, *, show_tool_icon_always=False, tool_key=None
    ) -> None:
        """

        :param context:
        :param layout:
        :param show_tool_icon_always:
        :param tool_key:
        """

    @classmethod
    def draw_cls(cls, layout, context, detect_layout=True, scale_y=1.75) -> None:
        """

        :param layout:
        :param context:
        :param detect_layout:
        :param scale_y:
        """

    @staticmethod
    def draw_fallback_tool_items(layout, context) -> None:
        """

        :param layout:
        :param context:
        """

    @staticmethod
    def draw_fallback_tool_items_for_pie_menu(layout, context) -> None:
        """

        :param layout:
        :param context:
        """

    @classmethod
    def keymap_ui_hierarchy(cls, context_mode) -> None:
        """

        :param context_mode:
        """

    @classmethod
    def register(cls) -> None: ...
    @classmethod
    def register_ensure(cls) -> None: ...
    @staticmethod
    def tool_active_from_context(context) -> None:
        """

        :param context:
        """

    @classmethod
    def tools_all(cls) -> None: ...
    @classmethod
    def tools_from_context(cls, context, mode=None) -> None:
        """

        :param context:
        :param mode:
        """

class WM_MT_toolsystem_submenu(_bpy_types.Menu):
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> None: ...
    def bl_rna_get_subclass_py(self) -> None: ...
    def draw(self, context) -> None:
        """

        :param context:
        """

def activate_by_id(context, space_type, idname, *, as_fallback=False) -> None: ...
def activate_by_id_or_cycle(
    context, space_type, idname, *, offset=1, as_fallback=False
) -> None: ...
def description_from_id(context, space_type, idname, *, use_operator=True) -> None: ...
def item_from_flat_index(context, space_type, index) -> None: ...
def item_from_id(context, space_type, idname) -> None: ...
def item_from_id_active(context, space_type, idname) -> None: ...
def item_from_id_active_with_group(context, space_type, idname) -> None: ...
def item_from_index_active(context, space_type, index) -> None: ...
def item_group_from_id(context, space_type, idname, *, coerce=False) -> None: ...
def keymap_from_id(context, space_type, idname) -> None: ...



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def delete_metaelems(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    confirm: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete selected metaball element(s)

    :param confirm: Confirm, Prompt for confirmation (optional)
    :return: Result of the operator call.
    """

def duplicate_metaelems(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Duplicate selected metaball element(s)

    :return: Result of the operator call.
    """

def duplicate_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    MBALL_OT_duplicate_metaelems: dict[str, typing.Any] | None = {},
    TRANSFORM_OT_translate: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Make copies of the selected metaball elements and move them

    :param MBALL_OT_duplicate_metaelems: Duplicate Metaball Elements, Duplicate selected metaball element(s) (optional, `bpy.ops.mball.duplicate_metaelems` keyword arguments)
    :param TRANSFORM_OT_translate: Move, Move selected items (optional, `bpy.ops.transform.translate` keyword arguments)
    :return: Result of the operator call.
    """

def hide_metaelems(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    unselected: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide (un)selected metaball element(s)

    :param unselected: Unselected, Hide unselected rather than selected (optional)
    :return: Result of the operator call.
    """

def reveal_metaelems(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    select: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reveal all hidden metaball elements

    :param select: Select, (optional)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Change selection of all metaball elements

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_random_metaelems(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    ratio: float | None = 0.5,
    seed: int | None = 0,
    action: typing.Literal["SELECT", "DESELECT"] | None = "SELECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Randomly select metaball elements

        :param ratio: Ratio, Portion of items to select randomly (in [0, 1], optional)
        :param seed: Random Seed, Seed for the random number generator (in [0, inf], optional)
        :param action: Action, Selection action to execute (optional)

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.
        :return: Result of the operator call.
    """

def select_similar(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["TYPE", "RADIUS", "STIFFNESS", "ROTATION"] | None = "TYPE",
    threshold: float | None = 0.1,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select similar metaballs by property types

    :param type: Type, (optional)
    :param threshold: Threshold, (in [0, inf], optional)
    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums
import mathutils

def add_feather_vertex(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    location: collections.abc.Sequence[float] | mathutils.Vector | None = (0.0, 0.0),
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add vertex to feather

    :param location: Location, Location of vertex in normalized space (array of 2 items, in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def add_feather_vertex_slide(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    MASK_OT_add_feather_vertex: dict[str, typing.Any] | None = {},
    MASK_OT_slide_point: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add new vertex to feather and slide it

    :param MASK_OT_add_feather_vertex: Add Feather Vertex, Add vertex to feather (optional, `bpy.ops.mask.add_feather_vertex` keyword arguments)
    :param MASK_OT_slide_point: Slide Point, Slide control points (optional, `bpy.ops.mask.slide_point` keyword arguments)
    :return: Result of the operator call.
    """

def add_vertex(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    location: collections.abc.Sequence[float] | mathutils.Vector | None = (0.0, 0.0),
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add vertex to active spline

    :param location: Location, Location of vertex in normalized space (array of 2 items, in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def add_vertex_slide(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    MASK_OT_add_vertex: dict[str, typing.Any] | None = {},
    MASK_OT_slide_point: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add new vertex and slide it

    :param MASK_OT_add_vertex: Add Vertex, Add vertex to active spline (optional, `bpy.ops.mask.add_vertex` keyword arguments)
    :param MASK_OT_slide_point: Slide Point, Slide control points (optional, `bpy.ops.mask.slide_point` keyword arguments)
    :return: Result of the operator call.
    """

def copy_splines(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy the selected splines to the internal clipboard

    :return: Result of the operator call.
    """

def cyclic_toggle(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Toggle cyclic for selected splines

    :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    confirm: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete selected control points or splines

    :param confirm: Confirm, Prompt for confirmation (optional)
    :return: Result of the operator call.
    """

def duplicate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Duplicate selected control points and segments between them

    :return: Result of the operator call.
    """

def duplicate_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    MASK_OT_duplicate: dict[str, typing.Any] | None = {},
    TRANSFORM_OT_translate: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Duplicate mask and move

    :param MASK_OT_duplicate: Duplicate Mask, Duplicate selected control points and segments between them (optional, `bpy.ops.mask.duplicate` keyword arguments)
    :param TRANSFORM_OT_translate: Move, Move selected items (optional, `bpy.ops.transform.translate` keyword arguments)
    :return: Result of the operator call.
    """

def feather_weight_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset the feather weight to zero

    :return: Result of the operator call.
    """

def handle_type_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["AUTO", "VECTOR", "ALIGNED", "ALIGNED_DOUBLESIDE", "FREE"]
    | None = "AUTO",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set type of handles for selected control points

    :param type: Type, Spline type (optional)
    :return: Result of the operator call.
    """

def hide_view_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    select: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reveal temporarily hidden mask layers

    :param select: Select, (optional)
    :return: Result of the operator call.
    """

def hide_view_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    unselected: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Temporarily hide mask layers

    :param unselected: Unselected, Hide unselected rather than selected layers (optional)
    :return: Result of the operator call.
    """

def layer_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    direction: typing.Literal["UP", "DOWN"] | None = "UP",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move the active layer up/down in the list

    :param direction: Direction, Direction to move the active layer (optional)
    :return: Result of the operator call.
    """

def layer_new(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add new mask layer for masking

    :param name: Name, Name of new mask layer (optional, never None)
    :return: Result of the operator call.
    """

def layer_remove(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove mask layer

    :return: Result of the operator call.
    """

def move_to_layer(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    target_layer_name: str | None = "",
    add_new_layer: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move the active spline to layer

    :param target_layer_name: Name, Target Mask Layer (optional, never None)
    :param add_new_layer: New Layer, Move selection to a new layer (optional)
    :return: Result of the operator call.
    """

def new(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Create new mask

    :param name: Name, Name of new mask (optional, never None)
    :return: Result of the operator call.
    """

def normals_make_consistent(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Recalculate the direction of selected handles

    :return: Result of the operator call.
    """

def parent_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear the masks parenting

    :return: Result of the operator call.
    """

def parent_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set the masks parenting

    :return: Result of the operator call.
    """

def paste_splines(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Paste splines from the internal clipboard

    :return: Result of the operator call.
    """

def primitive_circle_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    size: float | None = 100.0,
    location: collections.abc.Sequence[float] | mathutils.Vector | None = (0.0, 0.0),
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add new circle-shaped spline

    :param size: Size, Size of new primitive (in [-inf, inf], optional)
    :param location: Location, Location of new primitive (array of 2 items, in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def primitive_square_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    size: float | None = 100.0,
    location: collections.abc.Sequence[float] | mathutils.Vector | None = (0.0, 0.0),
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add new square-shaped spline

    :param size: Size, Size of new primitive (in [-inf, inf], optional)
    :param location: Location, Location of new primitive (array of 2 items, in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def select(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    extend: bool | None = False,
    deselect: bool | None = False,
    toggle: bool | None = False,
    deselect_all: bool | None = False,
    select_passthrough: bool | None = False,
    location: collections.abc.Sequence[float] | mathutils.Vector | None = (0.0, 0.0),
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select spline points

    :param extend: Extend, Extend selection instead of deselecting everything first (optional)
    :param deselect: Deselect, Remove from selection (optional)
    :param toggle: Toggle Selection, Toggle the selection (optional)
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor (optional)
    :param select_passthrough: Only Select Unselected, Ignore the select action when the element is already selected (optional)
    :param location: Location, Location of vertex in normalized space (array of 2 items, in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Change selection of all curve points

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_box(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    xmin: int | None = 0,
    xmax: int | None = 0,
    ymin: int | None = 0,
    ymax: int | None = 0,
    wait_for_input: bool | None = True,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select curve points using box selection

        :param xmin: X Min, (in [-inf, inf], optional)
        :param xmax: X Max, (in [-inf, inf], optional)
        :param ymin: Y Min, (in [-inf, inf], optional)
        :param ymax: Y Max, (in [-inf, inf], optional)
        :param wait_for_input: Wait for Input, (optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :return: Result of the operator call.
    """

def select_circle(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    x: int | None = 0,
    y: int | None = 0,
    radius: int | None = 25,
    wait_for_input: bool | None = True,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select curve points using circle selection

        :param x: X, (in [-inf, inf], optional)
        :param y: Y, (in [-inf, inf], optional)
        :param radius: Radius, (in [1, inf], optional)
        :param wait_for_input: Wait for Input, (optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :return: Result of the operator call.
    """

def select_lasso(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    path=None,
    use_smooth_stroke: bool | None = False,
    smooth_stroke_factor: float | None = 0.75,
    smooth_stroke_radius: int | None = 35,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select curve points using lasso selection

        :param path: Path, (optional)
        :param use_smooth_stroke: Stabilize Stroke, Selection lags behind mouse and follows a smoother path (optional)
        :param smooth_stroke_factor: Smooth Stroke Factor, Higher values give a smoother stroke (in [0.5, 0.99], optional)
        :param smooth_stroke_radius: Smooth Stroke Radius, Minimum distance from last point before selection continues (in [10, 200], optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :return: Result of the operator call.
    """

def select_less(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Deselect spline points at the boundary of each selection region

    :return: Result of the operator call.
    """

def select_linked(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select all curve points linked to already selected ones

    :return: Result of the operator call.
    """

def select_linked_pick(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    deselect: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """(De)select all points linked to the curve under the mouse cursor

    :param deselect: Deselect, (optional)
    :return: Result of the operator call.
    """

def select_more(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select more spline points connected to initial selection

    :return: Result of the operator call.
    """

def shape_key_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove mask shape keyframe for active mask layer at the current frame

    :return: Result of the operator call.
    """

def shape_key_feather_reset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset feather weights on all selected points animation values

    :return: Result of the operator call.
    """

def shape_key_insert(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Insert mask shape keyframe for active mask layer at the current frame

    :return: Result of the operator call.
    """

def shape_key_rekey(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    location: bool | None = True,
    feather: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Recalculate animation data on selected points for frames selected in the dopesheet

    :param location: Location, (optional)
    :param feather: Feather, (optional)
    :return: Result of the operator call.
    """

def slide_point(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    slide_feather: bool | None = False,
    is_new_point: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Slide control points

    :param slide_feather: Slide Feather, First try to slide feather instead of vertex (optional)
    :param is_new_point: Slide New Point, Newly created vertex is being slid (optional)
    :return: Result of the operator call.
    """

def slide_spline_curvature(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Slide a point on the spline to define its curvature

    :return: Result of the operator call.
    """

def switch_direction(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Switch direction of selected splines

    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def action_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Change the active action used

    :param action: Action, (optional)
    :return: Result of the operator call.
    """

def animdata_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "CLEAR_ANIMDATA", "SET_ACT", "CLEAR_ACT", "REFRESH_DRIVERS", "CLEAR_DRIVERS"
    ]
    | None = "CLEAR_ANIMDATA",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

        :param type: Animation Operation, (optional)

    CLEAR_ANIMDATA
    Clear Animation Data -- Remove this animation data container.

    SET_ACT
    Set Action.

    CLEAR_ACT
    Unlink Action.

    REFRESH_DRIVERS
    Refresh Drivers.

    CLEAR_DRIVERS
    Clear Drivers.
        :return: Result of the operator call.
    """

def clear_filter(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear the search filter

    :return: Result of the operator call.
    """

def collection_color_tag_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    color: Literal[bpy.stub_internal.rna_enums.CollectionColorItems] | None = "NONE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set a color tag for the selected collections

    :param color: Color Tag, (optional)
    :return: Result of the operator call.
    """

def collection_disable(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Disable viewport display in the view layers

    :return: Result of the operator call.
    """

def collection_disable_render(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Do not render this collection

    :return: Result of the operator call.
    """

def collection_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag to move to collection in Outliner

    :return: Result of the operator call.
    """

def collection_duplicate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Recursively duplicate the collection, all its children, objects and object data

    :return: Result of the operator call.
    """

def collection_duplicate_linked(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Recursively duplicate the collection, all its children and objects, with linked object data

    :return: Result of the operator call.
    """

def collection_enable(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Enable viewport display in the view layers

    :return: Result of the operator call.
    """

def collection_enable_render(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Render the collection

    :return: Result of the operator call.
    """

def collection_exclude_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Include collection in the active view layer

    :return: Result of the operator call.
    """

def collection_exclude_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Exclude collection from the active view layer

    :return: Result of the operator call.
    """

def collection_hide(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide the collection in this view layer

    :return: Result of the operator call.
    """

def collection_hide_inside(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide all the objects and collections inside the collection

    :return: Result of the operator call.
    """

def collection_hierarchy_delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete selected collection hierarchies

    :return: Result of the operator call.
    """

def collection_holdout_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear masking of collection in the active view layer

    :return: Result of the operator call.
    """

def collection_holdout_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Mask collection in the active view layer

    :return: Result of the operator call.
    """

def collection_indirect_only_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear collection contributing only indirectly in the view layer

    :return: Result of the operator call.
    """

def collection_indirect_only_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set collection to only contribute indirectly (through shadows and reflections) in the view layer

    :return: Result of the operator call.
    """

def collection_instance(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Instance selected collections to active scene

    :return: Result of the operator call.
    """

def collection_isolate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    extend: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide all but this collection and its parents

    :param extend: Extend, Extend current visible collections (optional)
    :return: Result of the operator call.
    """

def collection_link(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Link selected collections to active scene

    :return: Result of the operator call.
    """

def collection_new(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    nested: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a new collection inside selected collection

    :param nested: Nested, Add as child of selected collection (optional)
    :return: Result of the operator call.
    """

def collection_objects_deselect(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Deselect objects in collection

    :return: Result of the operator call.
    """

def collection_objects_select(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select objects in collection

    :return: Result of the operator call.
    """

def collection_show(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Show the collection in this view layer

    :return: Result of the operator call.
    """

def collection_show_inside(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Show all the objects and collections inside the collection

    :return: Result of the operator call.
    """

def constraint_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["ENABLE", "DISABLE", "DELETE"] | None = "ENABLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param type: Constraint Operation, (optional)
    :return: Result of the operator call.
    """

def data_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["DEFAULT"] | None = "DEFAULT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param type: Data Operation, (optional)
    :return: Result of the operator call.
    """

def datastack_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy or reorder modifiers, constraints, and effects

    :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    hierarchy: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete selected objects and collections

    :param hierarchy: Hierarchy, Delete child objects and collections (optional)
    :return: Result of the operator call.
    """

def drivers_add_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add drivers to selected items

    :return: Result of the operator call.
    """

def drivers_delete_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete drivers assigned to selected items

    :return: Result of the operator call.
    """

def expanded_toggle(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Expand/Collapse all items

    :return: Result of the operator call.
    """

def hide(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide selected objects and collections

    :return: Result of the operator call.
    """

def highlight_update(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Update the item highlight based on the current mouse position

    :return: Result of the operator call.
    """

def id_copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy the selected data-blocks to the internal clipboard

    :return: Result of the operator call.
    """

def id_delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete the ID under cursor

    :return: Result of the operator call.
    """

def id_linked_relocate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Replace the active linked ID (and its dependencies if any) by another one, from the same or a different library

    :return: Result of the operator call.
    """

def id_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "UNLINK",
        "LOCAL",
        "SINGLE",
        "DELETE",
        "REMAP",
        "COPY",
        "PASTE",
        "ADD_FAKE",
        "CLEAR_FAKE",
        "RENAME",
        "SELECT_LINKED",
    ]
    | None = "UNLINK",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """General data-block management operations

        :param type: ID Data Operation, (optional)

    UNLINK
    Unlink.

    LOCAL
    Make Local.

    SINGLE
    Make Single User.

    DELETE
    Delete.

    REMAP
    Remap Users -- Make all users of selected data-blocks to use instead current (clicked) one.

    COPY
    Copy.

    PASTE
    Paste.

    ADD_FAKE
    Add Fake User -- Ensure data-block gets saved even if it isnt in use (e.g. for motion and material libraries).

    CLEAR_FAKE
    Clear Fake User.

    RENAME
    Rename.

    SELECT_LINKED
    Select Linked.
        :return: Result of the operator call.
    """

def id_paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Paste data-blocks from the internal clipboard

    :return: Result of the operator call.
    """

def id_remap(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    id_type: Literal[bpy.stub_internal.rna_enums.IdTypeItems] | None = "OBJECT",
    old_id: int | None = 0,
    new_id: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param id_type: ID Type, (optional)
    :param old_id: Old ID, Old IDs session uid to remap data from (in [-inf, inf], optional)
    :param new_id: New ID, New IDs session uid to remap all selected IDs users to (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def item_activate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    extend: bool | None = False,
    extend_range: bool | None = False,
    deselect_all: bool | None = False,
    recurse: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Handle mouse clicks to select and activate items

    :param extend: Extend, Extend selection for activation (optional)
    :param extend_range: Extend Range, Select a range from active element (optional)
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor (optional)
    :param recurse: Recurse, Select objects recursively from active element (optional)
    :return: Result of the operator call.
    """

def item_drag_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag and drop element to another place

    :return: Result of the operator call.
    """

def item_openclose(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    all: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Toggle whether item under cursor is open or closed

    :param all: All, Close or open all items (optional)
    :return: Result of the operator call.
    """

def item_rename(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    use_active: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Rename the active element

    :param use_active: Use Active, Rename the active item, rather than the one the mouse is over (optional)
    :return: Result of the operator call.
    """

def keyingset_add_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add selected items (blue-gray rows) to active Keying Set

    :return: Result of the operator call.
    """

def keyingset_remove_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove selected items (blue-gray rows) from active Keying Set

    :return: Result of the operator call.
    """

def lib_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["DELETE", "RELOCATE", "RELOAD"] | None = "DELETE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

        :param type: Library Operation, (optional)

    DELETE
    Delete -- Delete this library and all its items.

    RELOCATE
    Relocate -- Select a new path for this library, and reload all its data.

    RELOAD
    Reload -- Reload all data from this library.
        :return: Result of the operator call.
    """

def lib_relocate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Relocate the library under cursor

    :return: Result of the operator call.
    """

def liboverride_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "OVERRIDE_LIBRARY_CREATE_HIERARCHY",
        "OVERRIDE_LIBRARY_RESET",
        "OVERRIDE_LIBRARY_CLEAR_SINGLE",
    ]
    | None = "OVERRIDE_LIBRARY_CREATE_HIERARCHY",
    selection_set: typing.Literal["SELECTED", "CONTENT", "SELECTED_AND_CONTENT"]
    | None = "SELECTED",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Create, reset or clear library override hierarchies

        :param type: Library Override Operation, (optional)

    OVERRIDE_LIBRARY_CREATE_HIERARCHY
    Make -- Create a local override of the selected linked data-blocks, and their hierarchy of dependencies.

    OVERRIDE_LIBRARY_RESET
    Reset -- Reset the selected local overrides to their linked references values.

    OVERRIDE_LIBRARY_CLEAR_SINGLE
    Clear -- Delete the selected local overrides and relink their usages to the linked data-blocks if possible, else reset them and mark them as non editable.
        :param selection_set: Selection Set, Over which part of the tree items to apply the operation (optional)

    SELECTED
    Selected -- Apply the operation over selected data-blocks only.

    CONTENT
    Content -- Apply the operation over content of the selected items only (the data-blocks in their sub-tree).

    SELECTED_AND_CONTENT
    Selected & Content -- Apply the operation over selected data-blocks and all their dependencies.
        :return: Result of the operator call.
    """

def liboverride_troubleshoot_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "OVERRIDE_LIBRARY_RESYNC_HIERARCHY",
        "OVERRIDE_LIBRARY_RESYNC_HIERARCHY_ENFORCE",
        "OVERRIDE_LIBRARY_DELETE_HIERARCHY",
    ]
    | None = "OVERRIDE_LIBRARY_RESYNC_HIERARCHY",
    selection_set: typing.Literal["SELECTED", "CONTENT", "SELECTED_AND_CONTENT"]
    | None = "SELECTED",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Advanced operations over library override to help fix broken hierarchies

        :param type: Library Override Troubleshoot Operation, (optional)

    OVERRIDE_LIBRARY_RESYNC_HIERARCHY
    Resync -- Rebuild the selected local overrides from their linked references, as well as their hierarchies of dependencies.

    OVERRIDE_LIBRARY_RESYNC_HIERARCHY_ENFORCE
    Resync Enforce -- Rebuild the selected local overrides from their linked references, as well as their hierarchies of dependencies, enforcing these hierarchies to match the linked data (i.e. ignoring existing overrides on data-blocks pointer properties).

    OVERRIDE_LIBRARY_DELETE_HIERARCHY
    Delete -- Delete the selected local overrides (including their hierarchies of override dependencies) and relink their usages to the linked data-blocks.
        :param selection_set: Selection Set, Over which part of the tree items to apply the operation (optional)

    SELECTED
    Selected -- Apply the operation over selected data-blocks only.

    CONTENT
    Content -- Apply the operation over content of the selected items only (the data-blocks in their sub-tree).

    SELECTED_AND_CONTENT
    Selected & Content -- Apply the operation over selected data-blocks and all their dependencies.
        :return: Result of the operator call.
    """

def material_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag material to object in Outliner

    :return: Result of the operator call.
    """

def modifier_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["APPLY", "DELETE", "TOGVIS", "TOGREN"] | None = "APPLY",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param type: Modifier Operation, (optional)
    :return: Result of the operator call.
    """

def object_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["SELECT", "DESELECT", "SELECT_HIERARCHY", "REMAP", "RENAME"]
    | None = "SELECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

        :param type: Object Operation, (optional)

    SELECT
    Select.

    DESELECT
    Deselect.

    SELECT_HIERARCHY
    Select Hierarchy.

    REMAP
    Remap Users -- Make all users of selected data-blocks to use instead a new chosen one.

    RENAME
    Rename.
        :return: Result of the operator call.
    """

def operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Context menu for item operations

    :return: Result of the operator call.
    """

def orphans_manage(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Open a window to manage unused data

    :return: Result of the operator call.
    """

def orphans_purge(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    do_local_ids: bool | None = True,
    do_linked_ids: bool | None = True,
    do_recursive: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear all orphaned data-blocks without any users from the file

    :param do_local_ids: Local Data-blocks, Include unused local data-blocks into deletion (optional)
    :param do_linked_ids: Linked Data-blocks, Include unused linked data-blocks into deletion (optional)
    :param do_recursive: Recursive Delete, Recursively check for indirectly unused data-blocks, ensuring that no orphaned data-blocks remain after execution (optional)
    :return: Result of the operator call.
    """

def parent_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag to clear parent in Outliner

    :return: Result of the operator call.
    """

def parent_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag to parent in Outliner

    :return: Result of the operator call.
    """

def scene_drop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Drag object to scene in Outliner

    :return: Result of the operator call.
    """

def scene_operation(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["DELETE"] | None = "DELETE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Context menu for scene operations

    :param type: Scene Operation, (optional)
    :return: Result of the operator call.
    """

def scroll_page(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    up: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Scroll page up or down

    :param up: Up, Scroll up one page (optional)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Toggle the Outliner selection of items

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_box(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    tweak: bool | None = False,
    xmin: int | None = 0,
    xmax: int | None = 0,
    ymin: int | None = 0,
    ymax: int | None = 0,
    wait_for_input: bool | None = True,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Use box selection to select tree elements

        :param tweak: Tweak, Tweak gesture from empty space for box selection (optional)
        :param xmin: X Min, (in [-inf, inf], optional)
        :param xmax: X Max, (in [-inf, inf], optional)
        :param ymin: Y Min, (in [-inf, inf], optional)
        :param ymax: Y Max, (in [-inf, inf], optional)
        :param wait_for_input: Wait for Input, (optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :return: Result of the operator call.
    """

def select_walk(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    direction: typing.Literal["UP", "DOWN", "LEFT", "RIGHT"] | None = "UP",
    extend: bool | None = False,
    toggle_all: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Use walk navigation to select tree elements

    :param direction: Walk Direction, Select/Deselect element in this direction (optional)
    :param extend: Extend, Extend selection on walk (optional)
    :param toggle_all: Toggle All, Toggle open/close hierarchy (optional)
    :return: Result of the operator call.
    """

def show_active(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Open up the tree and adjust the view so that the active object is shown centered

    :return: Result of the operator call.
    """

def show_hierarchy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Open all object entries and close all others

    :return: Result of the operator call.
    """

def show_one_level(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    open: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Expand/collapse all entries by one level

    :param open: Open, Expand all entries one level deep (optional)
    :return: Result of the operator call.
    """

def start_filter(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Start entering filter text

    :return: Result of the operator call.
    """

def unhide_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Unhide all objects and collections

    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class SpellChecker:
    """A basic spell checker."""

    uimsgs: typing.Any

    def check(self, txt) -> None:
        """

        :param txt:
        """

    @classmethod
    def split_words(cls, text) -> None:
        """

        :param text:
        """
